use chrono::NaiveDateTime;
use diesel::prelude::*;
use serde_derive::Serialize;

#[derive(Identifiable, Queryable, Selectable, Insertable, Debug)]
#[diesel(table_name = crate::schema::post)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct Post {
    pub id: i64,
    pub description: String,
    pub posted_date: NaiveDateTime,
    pub crawled_date: NaiveDateTime,
}

#[derive(Identifiable, Queryable, Selectable, Associations, Debug)]
#[diesel(belongs_to(Post))]
#[diesel(table_name = crate::schema::image)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct Image {
    pub id: i32,
    pub post_id: i64,
    pub url: String,
}

#[derive(Insertable, Associations, Debug)]
#[diesel(belongs_to(Post))]
#[diesel(table_name = crate::schema::image)]
pub struct NewImage {
    pub post_id: i64,
    pub url: String,
}

#[derive(Identifiable, Queryable, Selectable, Associations, Debug)]
#[diesel(belongs_to(Post))]
#[diesel(table_name = crate::schema::video)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct Video {
    pub id: i32,
    pub post_id: i64,
    pub url: String,
}

#[derive(Insertable, Associations, Debug)]
#[diesel(belongs_to(Post))]
#[diesel(table_name = crate::schema::video)]
pub struct NewVideo {
    pub post_id: i64,
    pub url: String,
}

#[derive(Serialize)]
pub struct PostWithMedia {
    pub id: i64,
    pub description: String,
    pub posted_date: NaiveDateTime,
    pub crawled_date: NaiveDateTime,
    pub images: Vec<String>,
    pub videos: Vec<String>,
}

impl PostWithMedia {
    pub fn from_post(post: &Post) -> Self {
        PostWithMedia {
            id: post.id,
            description: post.description.clone(),
            posted_date: post.posted_date,
            crawled_date: post.crawled_date,
            images: vec![],
            videos: vec![],
        }
    }
}
